#include <18F8722.H>
#DEVICE ADC=10
#fuses H4,NOPUT, NOBROWNOUT, NOLVP, NOWRT, WDT512,NOPROTECT,NODEBUG,MCLR
// Fuses: LP,XT,HS,RC,EC,EC_IO,H4,RC_IO,PROTECT,NOPROTECT,IESO,NOIESO
// Fuses: NOBROWNOUT,BROWNOUT,WDT1,WDT2,WDT4,WDT8,WDT16,WDT32,WDT64
// Fuses: WDT128,WDT,NOWDT,BORV25,BORV27,BORV42,BORV45,PUT,NOPUT,CPD
// Fuses: NOCPD,NOSTVREN,STVREN,NODEBUG,DEBUG,NOLVP,LVP,WRT,NOWRT,CPB
// Fuses: NOCPB,EBTRB,NOEBTRB,EBTR,NOEBTR,CCP2E7,CCP2C1,WRTD,NOWRTD
// Fuses: WRTC,NOWRTC,WRTB,NOWRTB,FCMEN,NOFCMEN,INTRC,INTRC_IO
// Fuses: BROWNOUT_SW,BROWNOUT_NOSL,WDT256,WDT512,WDT1024,WDT2048
// Fuses: WDT4096,WDT8192,WDT16384,WDT32768,LPT1OSC,NOLPT1OSC,MCLR
// Fuses: NOMCLR,XINST,NOXINST,MCU,CPU,CPU_BB,EMCU,WAIT,NOWAIT,BW16
// Fuses: BW8,ABW8,ABW12,ABW16,ABW20,ECCPE,ECCPH

#use delay(clock=24000000)

#define WR      PIN_G2
#define RD      PIN_G1
#define TXE     PIN_B4
#define RXF     PIN_B2
#define LED		PIN_J5

int8 led_state;
int8 buffer[10];
int8 D_Data;
int8 D_Error;
int8 D_Presence;
int8 D_SC;
int8 packet_ready;
int8 d_pin;

void send(int8 thebyte);
int8 read_usb();
void receive_packet();
int8 Read_byte(void);
void D_Reset(void);
void Write_byte(int8 Data);
void make_low(int8 pin_to_use);
void make_high(int8 pin_to_use);
int8 get_line_in(int8 pin_to_use);
void setup_AD();
void run_AD();
void go_blink_fast();
void go_blink_slow();


//**********************************************************************
void go_blink_fast()
{
   int8 tt;
   while(1)
   {
   	for(tt=0; tt<20; tt++)
   	{
      	output_high(LED);
   		delay_ms(30);
      	output_low(LED);
   		delay_ms(30);
   	}
   }
}

//**********************************************************************
void go_blink_slow()
{
   int8 tt;
   while(1)
   {
   	for(tt=0; tt<10; tt++)
   	{
      	output_high(LED);
   		delay_ms(100);
      	output_low(LED);
   		delay_ms(100);
   	}
   }
}



//********************************************************************
void main()
{
   int8 databit, tt;
   int16 addr;


	//1111 1111 port A idle state: 7654 3210  1=input 0=output
	SET_TRIS_A( 0xff );//all input
	SET_TRIS_B( 0xff );
	SET_TRIS_C( 0xff );
	SET_TRIS_D( 0xff );
	SET_TRIS_E( 0xff );
	SET_TRIS_F( 0xff );
	SET_TRIS_G( 0xff );
	SET_TRIS_H( 0xff );
	SET_TRIS_J( 0xff );


	//setup_adc(ADC_CLOCK_INTERNAL);//disable the A/D converter
	setup_adc_ports( NO_ANALOGS );//default to all digital I/O
	setup_adc(ADC_OFF );//disable the A/D converter

	output_high(WR);//init USB WR
	output_high(RD);//init USB RD

	delay_ms(100);

	if(!input(RXF))//purge USB FIFO
	{
   		output_low(RD);
   		output_high(RD);
	}

	//flash LED
	for(tt=0; tt<15; tt++)
	{
   	output_high(LED);
		delay_ms(50);
   	output_low(LED);
		delay_ms(50);
	}



	while(1)
	{
		RESTART_WDT();

		//byte received in FIFO buffer...
		if(!input(RXF))
		{
			receive_packet();


			//if Return Board ID command...
			//**********************************************
			if(buffer[1] == 0xA7)
			{
				send('2');
				send('4');
				send('5');
				send('P');
				send('L');

				buffer[0] = 0;
				continue;
			}


			//if Setup A/D command...
			//**********************************************
			if(buffer[1] == 0xA8)
			{
				setup_AD();
				buffer[1] = 0;
				continue;
			}

			//if run A/D command...
			//**********************************************
			if(buffer[1] == 0xA9)
			{
				run_AD();
				buffer[1] = 0;
				continue;
			}

			//if Line out command...
			//**********************************************
			if(buffer[1] == 0xA6)
			{
				if(buffer[3])//if high requested
					make_high(buffer[2]);
				   else
					make_low(buffer[2]);//low requested
				buffer[1] = 0;
				continue;
			}

			//if Line in command...
			//**********************************************
			if(buffer[1] == 0xA5)
			{
				databit = get_line_in(buffer[2]);
				send(databit);

				buffer[1] = 0;
				continue;
			}

			//if Loopback test
			//**********************************************
			if(buffer[1] == 0xAF)
			{
				send(buffer[2]);

				buffer[1] = 0;
				continue;
			}

			//if EEPROM Read
			//**********************************************
			if(buffer[1] == 0xAA)
			{
				addr = ((buffer[2] << 8) | buffer[3]);
				send( read_EEPROM(addr) );

				buffer[1] = 0;
				continue;
			}

			//if EEPROM Write
			//**********************************************
			if(buffer[1] == 0xAB)
			{
				addr = ((buffer[2] << 8) | buffer[3]);
				write_eeprom(addr, buffer[4]);
				buffer[1] = 0;
				continue;
			}

			//if temperature start convert command
			//**********************************************
			if(buffer[1] == 0xAC)//start convert
			{
				d_pin = buffer[2];

			   D_Reset();
				if(D_Error < 99)
				{
					send(D_Error);//return error
					continue;
				}
				else
					send(0x63);//return no error (99)

				Write_byte(0xcc);//skip rom command

				Write_byte(0x44);//Convert temperatures
				buffer[1] = 0;
            
				continue;
			}

			//if temperature read command -
			//**********************************************
			if(buffer[1] == 0xAD)//READ DATA
			{
				d_pin = buffer[2];

			   D_Reset();
				Write_byte(0xcc);//skip rom command
				Write_byte(0xbe);//read scratch pad
				delay_us(5);

				for(tt=0; tt<9; tt++)
					send(Read_byte());
//go_blink_fast();

				buffer[1] = 0;
				continue;//probably not needed
			}


			//if Port A read
			//**********************************************
			if(buffer[1] == 0x55)
			{
				send( input_a() );
				buffer[1] = 0;

				continue;
			}

			//if Port A write
			//**********************************************
			if(buffer[1] == 0x56)
			{
				output_a(buffer[2]);
				buffer[1] = 0;
				continue;
			}

			//if Port C read
			//**********************************************
			if(buffer[1] == 0x59)
			{
				send( input_c() );
				buffer[1] = 0;

				continue;
			}

			//if Port C write
			//**********************************************
			if(buffer[1] == 0x5A)
			{
				output_c(buffer[2]);
				buffer[1] = 0;
				continue;
			}

			//if Port D read
			//**********************************************
			if(buffer[1] == 0x5B)
			{
				send( input_d() );
				buffer[1] = 0;

				continue;
			}

			//if Port D write
			//**********************************************
			if(buffer[1] == 0x5C)
			{
				output_d(buffer[2]);
				buffer[1] = 0;
				continue;
			}


			//if Port E read
			//**********************************************
			if(buffer[1] == 0x5D)
			{
				send( input_e() );
				buffer[1] = 0;

				continue;
			}

			//if Port E write
			//**********************************************
			if(buffer[1] == 0x5E)
			{
				output_e(buffer[2]);
				buffer[1] = 0;
				continue;
			}

			//if Port F read
			//**********************************************
			if(buffer[1] == 0x65)
			{
				send( input_f() );
				buffer[1] = 0;

				continue;
			}

			//if Port F write
			//**********************************************
			if(buffer[1] == 0x66)
			{
				output_f(buffer[2]);
				buffer[1] = 0;
				continue;
			}


			//if Port G read
			//**********************************************
			if(buffer[1] == 0x67)
			{
				send( input_g() );
				buffer[1] = 0;

				continue;
			}

			//if Port G write
			//**********************************************
			if(buffer[1] == 0x68)
			{
				output_g(buffer[2]);
				buffer[1] = 0;
				continue;
			}

			//if Port H read
			//**********************************************
			if(buffer[1] == 0x69)
			{
				send( input_h() );
				buffer[1] = 0;

				continue;
			}

			//if Port H write
			//**********************************************
			if(buffer[1] == 0x6A)
			{
				output_h(buffer[2]);
				buffer[1] = 0;
				continue;
			}

			//if Port J read
			//**********************************************
			if(buffer[1] == 0x6B)
			{
				send( input_j() );
				buffer[1] = 0;

				continue;
			}

			//if Port J write
			//**********************************************
			if(buffer[1] == 0x6C)
			{
				output_j(buffer[2]);
				buffer[1] = 0;
				continue;
			}


		}//end of if(!RXF)

	}//end of while(1)
}//end of main()




/*
// ADC Functions: SETUP_ADC(), SETUP_ADC_PORTS() (aka SETUP_PORT_A),
//                SET_ADC_CHANNEL(), READ_ADC()
// Constants used for SETUP_ADC() are:
#define ADC_OFF               0           // ADC Off
#define ADC_CLOCK_DIV_2   0x100
#define ADC_CLOCK_DIV_4    0x04
#define ADC_CLOCK_DIV_8    0x01
#define ADC_CLOCK_DIV_16   0x05
#define ADC_CLOCK_DIV_32   0x02
#define ADC_CLOCK_DIV_64   0x06
#define ADC_CLOCK_INTERNAL 0x07           // Internal 2-6us
// The following may be OR'ed in with the above using |
#define ADC_TAD_MUL_0      0x00
#define ADC_TAD_MUL_2      0x08
#define ADC_TAD_MUL_4      0x10
#define ADC_TAD_MUL_6      0x18
#define ADC_TAD_MUL_8      0x20
#define ADC_TAD_MUL_12     0x28
#define ADC_TAD_MUL_16     0x30
#define ADC_TAD_MUL_20     0x38

// Constants used in SETUP_ADC_PORTS() are:
#define NO_ANALOGS  0x0F  // None
#define ALL_ANALOG  0x00  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4 F5 F6 H4 H5 H6 H7 
#define AN0_TO_AN13 0x01  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4 F5 F6 H4 H5       
#define AN0_TO_AN12 0x02  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4 F5 F6 H4          
#define AN0_TO_AN11 0x03  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4 F5 F6             
#define AN0_TO_AN10 0x04  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4 F5                 
#define AN0_TO_AN9  0x05  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4                    
#define AN0_TO_AN8  0x06  // A0 A1 A2 A3 A5 F0 F1 F2 F3                       
#define AN0_TO_AN7  0x07  // A0 A1 A2 A3 A5 F0 F1 F2                         
#define AN0_TO_AN6  0x08  // A0 A1 A2 A3 A5 F0 F1
#define AN0_TO_AN5  0x09  // A0 A1 A2 A3 A5 F0
#define AN0_TO_AN4  0x0A  // A0 A1 A2 A3 A5
#define AN0_TO_AN3  0x0B  // A0 A1 A2 A3
#define AN0_TO_AN2  0x0C  // A0 A1 A2
#define AN0_TO_AN1  0x0D  // A0 A1
#define AN0         0x0E  // A0
*/

//**********************************************************************
void setup_AD()//0xA8
{
	int8 xx;

   switch (buffer[2])//selects which I/Os on port A are analog in and which are digital I/O
   {
   case 0:
   	setup_adc_ports( 0 ); //See above for description: 0=ALL_ANALOG: A0 A1 A2 A3 A5 E0 E1 E2
    break;
   case 1:
   	setup_adc_ports( 1 );
    break;
   case 2:
   	setup_adc_ports( 2 );
    break;
   case 3:
   	setup_adc_ports( 3 );
    break;
   case 4:
   	setup_adc_ports( 4 );
    break;
   case 5:
   	setup_adc_ports( 5 );
    break;
   case 6:
   	setup_adc_ports( 6 );
    break;
   case 7:
   	setup_adc_ports( 7 );
    break;
   case 8:
   	setup_adc_ports( 8 );
    break;
   case 9:
   	setup_adc_ports( 9 );
    break;
   case 0xa:
   	setup_adc_ports( 0x0a );
    break;
   case 0xb:
   	setup_adc_ports( 0x0b );
    break;
   case 0xc:
   	setup_adc_ports( 0x0c );
    break;
   case 0xd:
   	setup_adc_ports( 0x0d );
    break;
   case 0xe:
   	setup_adc_ports( 0x0e );
    break;
   case 0xf:
   	setup_adc_ports( 0x0f );
    break;
   default:
   	setup_adc_ports( 0 );
   }

   switch (buffer[3])//selects A/D clock source
   {
   case 0:
	 setup_adc( ADC_OFF );
     break;
   case 1:
	 setup_adc( ADC_CLOCK_DIV_8 );
     break;
   case 2:
	 setup_adc( ADC_CLOCK_DIV_32 );
     break;
   case 4:
	 setup_adc( ADC_CLOCK_DIV_4 );
     break;
   case 5:
	 setup_adc( ADC_CLOCK_DIV_16 );
     break;
   case 6:
	 setup_adc( ADC_CLOCK_DIV_64 );
     break;
   case 7:
	 setup_adc( ADC_CLOCK_INTERNAL );
     break;
   case 8:
	 setup_adc( ADC_CLOCK_DIV_2 );
     break;
   }

//	if(buffer[3] != 0)//if at least one channel is active
//      get_line_in(buffer[4]);//set TRIS to input so A/D doesn't convert output state
}



//**********************************************************************
void run_AD()//0xA9
{
	int16 dataword;

	set_adc_channel( buffer[2] );
	delay_us(10);
	dataword = read_adc();

	//return as two bytes to host - LSByte first
	send((int8)(dataword & 0x00ff));//LSByte
	send((int8)(dataword >> 8));//MSByte

}


//**********************************************************************
int8 get_line_in(int8 pin_to_use)
{
	int8 data;

	switch(pin_to_use)
	{
        case 44  : data = (int8)input(PIN_A0); break;
        case 45  : data = (int8)input(PIN_A1); break;
        case 46  : data = (int8)input(PIN_A2); break;
        case 47  : data = (int8)input(PIN_A3); break;
        case 48  : data = (int8)input(PIN_A4); break;
        case 49  : data = (int8)input(PIN_A5); break;

		case 52  : data = (int8)input(PIN_B0); break;
		case 53  : data = (int8)input(PIN_B1); break;
		case 54  : data = (int8)input(PIN_B2); break;
		case 55  : data = (int8)input(PIN_B3); break;
		case 56  : data = (int8)input(PIN_B4); break;
		case 57  : data = (int8)input(PIN_B5); break;
		case 58  : data = (int8)input(PIN_B6); break;
		case 59  : data = (int8)input(PIN_B7); break;

        case 60  : data = (int8)input(PIN_C0); break;
        case 61  : data = (int8)input(PIN_C1); break;
        case 62  : data = (int8)input(PIN_C2); break;
        case 63  : data = (int8)input(PIN_C3); break;
        case 64  : data = (int8)input(PIN_C4); break;
        case 65  : data = (int8)input(PIN_C5); break;
        case 66  : data = (int8)input(PIN_C6); break;
        case 67  : data = (int8)input(PIN_C7); break;

		case 68  : data = (int8)input(PIN_D0); break;
		case 69  : data = (int8)input(PIN_D1); break;
		case 70  : data = (int8)input(PIN_D2); break;
		case 71  : data = (int8)input(PIN_D3); break;
		case 72  : data = (int8)input(PIN_D4); break;
		case 73  : data = (int8)input(PIN_D5); break;
		case 74  : data = (int8)input(PIN_D6); break;
		case 75  : data = (int8)input(PIN_D7); break;

		case 76  : data = (int8)input(PIN_E0); break;
		case 77  : data = (int8)input(PIN_E1); break;
		case 78  : data = (int8)input(PIN_E2); break;
		case 79  : data = (int8)input(PIN_E3); break;
		case 80  : data = (int8)input(PIN_E4); break;
		case 81  : data = (int8)input(PIN_E5); break;
		case 82  : data = (int8)input(PIN_E6); break;
		case 83  : data = (int8)input(PIN_E7); break;

		case 84  : data = (int8)input(PIN_F0); break;
		case 85  : data = (int8)input(PIN_F1); break;
		case 86  : data = (int8)input(PIN_F2); break;
		case 87  : data = (int8)input(PIN_F3); break;
		case 88  : data = (int8)input(PIN_F4); break;
		case 89  : data = (int8)input(PIN_F5); break;
		case 90  : data = (int8)input(PIN_F6); break;
		case 91  : data = (int8)input(PIN_F7); break;

		case 92  : data = (int8)input(PIN_G0); break;
		//case 93  : data = (int8)input(PIN_G1); break;
		//case 94  : data = (int8)input(PIN_G2); break;
		case 95  : data = (int8)input(PIN_G3); break;
		//case 96  : data = (int8)input(PIN_G4); break;

		case 100 : data = (int8)input(PIN_H0); break;
		case 101 : data = (int8)input(PIN_H1); break;
		case 102 : data = (int8)input(PIN_H2); break;
		case 103 : data = (int8)input(PIN_H3); break;
		case 104 : data = (int8)input(PIN_H4); break;
		case 105 : data = (int8)input(PIN_H5); break;
		case 106 : data = (int8)input(PIN_H6); break;
		case 107 : data = (int8)input(PIN_H7); break;

		case 108 : data = (int8)input(PIN_J0); break;
		case 109 : data = (int8)input(PIN_J1); break;
		case 110 : data = (int8)input(PIN_J2); break;
		case 111 : data = (int8)input(PIN_J3); break;
		case 112 : data = (int8)input(PIN_J4); break;
		case 113 : data = (int8)input(PIN_J5); break;
		case 114 : data = (int8)input(PIN_J6); break;
		case 115 : data = (int8)input(PIN_J7); break;
	}

	return data;
}




//**********************************************************************
void make_high(int8 pin_to_use)
{
   switch(pin_to_use)
   {
        case 44  : output_high(PIN_A0); break;
        case 45  : output_high(PIN_A1); break;
        case 46  : output_high(PIN_A2); break;
        case 47  : output_high(PIN_A3); break;
        case 48  : output_high(PIN_A4); break;
        case 49  : output_high(PIN_A5); break;

		case 52  : output_high(PIN_B0); break;
		case 53  : output_high(PIN_B1); break;
		//case 54  : output_high(PIN_B2); break;
		case 55  : output_high(PIN_B3); break;
		//case 56  : output_high(PIN_B4); break;
		case 57  : output_high(PIN_B5); break;
		case 58  : output_high(PIN_B6); break;
		case 59  : output_high(PIN_B7); break;

        case 60  : output_high(PIN_C0); break;
        case 61  : output_high(PIN_C1); break;
        case 62  : output_high(PIN_C2); break;
        case 63  : output_high(PIN_C3); break;
        case 64  : output_high(PIN_C4); break;
        case 65  : output_high(PIN_C5); break;
        case 66  : output_high(PIN_C6); break;
        case 67  : output_high(PIN_C7); break;

		case 68  : output_high(PIN_D0); break;
		case 69  : output_high(PIN_D1); break;
		case 70  : output_high(PIN_D2); break;
		case 71  : output_high(PIN_D3); break;
		case 72  : output_high(PIN_D4); break;
		case 73  : output_high(PIN_D5); break;
		case 74  : output_high(PIN_D6); break;
		case 75  : output_high(PIN_D7); break;

		case 76  : output_high(PIN_E0); break;
		case 77  : output_high(PIN_E1); break;
		case 78  : output_high(PIN_E2); break;
		case 79  : output_high(PIN_E3); break;
		case 80  : output_high(PIN_E4); break;
		case 81  : output_high(PIN_E5); break;
		case 82  : output_high(PIN_E6); break;
		case 83  : output_high(PIN_E7); break;

		case 84  : output_high(PIN_F0); break;
		case 85  : output_high(PIN_F1); break;
		case 86  : output_high(PIN_F2); break;
		case 87  : output_high(PIN_F3); break;
		case 88  : output_high(PIN_F4); break;
		case 89  : output_high(PIN_F5); break;
		case 90  : output_high(PIN_F6); break;
		case 91  : output_high(PIN_F7); break;

		case 92  : output_high(PIN_G0); break;
		//case 93  : output_high(PIN_G1); break;
		//case 94  : output_high(PIN_G2); break;
		case 95  : output_high(PIN_G3); break;
		//case 96  : output_high(PIN_G4); break;

		case 100 : output_high(PIN_H0); break;
		case 101 : output_high(PIN_H1); break;
		case 102 : output_high(PIN_H2); break;
		case 103 : output_high(PIN_H3); break;
		case 104 : output_high(PIN_H4); break;
		case 105 : output_high(PIN_H5); break;
		case 106 : output_high(PIN_H6); break;
		case 107 : output_high(PIN_H7); break;

		case 108 : output_high(PIN_J0); break;
		case 109 : output_high(PIN_J1); break;
		case 110 : output_high(PIN_J2); break;
		case 111 : output_high(PIN_J3); break;
		case 112 : output_high(PIN_J4); break;
		case 113 : output_high(PIN_J5); break;
		case 114 : output_high(PIN_J6); break;
		case 115 : output_high(PIN_J7); break;
   }
}

//**********************************************************************
void make_low(int8 pin_to_use)
{
   switch(pin_to_use)
   {
        case 44  : output_low(PIN_A0); break;
        case 45  : output_low(PIN_A1); break;
        case 46  : output_low(PIN_A2); break;
        case 47  : output_low(PIN_A3); break;
        case 48  : output_low(PIN_A4); break;
        case 49  : output_low(PIN_A5); break;

		case 52  : output_low(PIN_B0); break;
		case 53  : output_low(PIN_B1); break;
		//case 54  : output_low(PIN_B2); break;
		case 55  : output_low(PIN_B3); break;
		//case 56  : output_low(PIN_B4); break;
		case 57  : output_low(PIN_B5); break;
		case 58  : output_low(PIN_B6); break;
		case 59  : output_low(PIN_B7); break;

        case 60  : output_low(PIN_C0); break;
        case 61  : output_low(PIN_C1); break;
        case 62  : output_low(PIN_C2); break;
        case 63  : output_low(PIN_C3); break;
        case 64  : output_low(PIN_C4); break;
        case 65  : output_low(PIN_C5); break;
        case 66  : output_low(PIN_C6); break;
        case 67  : output_low(PIN_C7); break;

		case 68  : output_low(PIN_D0); break;
		case 69  : output_low(PIN_D1); break;
		case 70  : output_low(PIN_D2); break;
		case 71  : output_low(PIN_D3); break;
		case 72  : output_low(PIN_D4); break;
		case 73  : output_low(PIN_D5); break;
		case 74  : output_low(PIN_D6); break;
		case 75  : output_low(PIN_D7); break;

		case 76  : output_low(PIN_E0); break;
		case 77  : output_low(PIN_E1); break;
		case 78  : output_low(PIN_E2); break;
		case 79  : output_low(PIN_E3); break;
		case 80  : output_low(PIN_E4); break;
		case 81  : output_low(PIN_E5); break;
		case 82  : output_low(PIN_E6); break;
		case 83  : output_low(PIN_E7); break;

		case 84  : output_low(PIN_F0); break;
		case 85  : output_low(PIN_F1); break;
		case 86  : output_low(PIN_F2); break;
		case 87  : output_low(PIN_F3); break;
		case 88  : output_low(PIN_F4); break;
		case 89  : output_low(PIN_F5); break;
		case 90  : output_low(PIN_F6); break;
		case 91  : output_low(PIN_F7); break;

		case 92  : output_low(PIN_G0); break;
		//case 93  : output_low(PIN_G1); break;
		//case 94  : output_low(PIN_G2); break;
		case 95  : output_low(PIN_G3); break;
		//case 96  : output_low(PIN_G4); break;

		case 100 : output_low(PIN_H0); break;
		case 101 : output_low(PIN_H1); break;
		case 102 : output_low(PIN_H2); break;
		case 103 : output_low(PIN_H3); break;
		case 104 : output_low(PIN_H4); break;
		case 105 : output_low(PIN_H5); break;
		case 106 : output_low(PIN_H6); break;
		case 107 : output_low(PIN_H7); break;

		case 108 : output_low(PIN_J0); break;
		case 109 : output_low(PIN_J1); break;
		case 110 : output_low(PIN_J2); break;
		case 111 : output_low(PIN_J3); break;
		case 112 : output_low(PIN_J4); break;
		case 113 : output_low(PIN_J5); break;
		case 114 : output_low(PIN_J6); break;
		case 115 : output_low(PIN_J7); break;
   }
}




//**********************************************************************
int8 Read_byte()//Read a byte from the DS18S20
{
	int8 data;
	int8 count;

	data=0;
	for(count=8; count>0; count--)
	{
		//Write the bit to the port
		make_low(d_pin);

		delay_us(2);

		//Release port for reading
		get_line_in(d_pin);

		//delay to center of timeslot
		delay_us(10);

		D_Data=get_line_in(d_pin); //Read the data bit in

		//Finish the timeslot
		delay_us(60);

		//Put data into the byte
		data = data >> 1;  //Shift previous bits across
		if(D_Data==1)      //If data was high then add high bit to data
			data +=0x80;

		//Delay Between Bits
		delay_us(10);
	}

	//rise time plus min space
	delay_us(10);

	return(data);
}






//**********************************************************************
void D_Reset()//Reset the DS18S20
{
	int8 count;

	D_Error=99; //-- Reset the status bit to no error

	get_line_in(d_pin);

	//Start the reset Pulse
	make_low(d_pin);

	delay_us(480);

	//release the line
	get_line_in(d_pin);

	//delay to about the center of the presence pulse
	delay_us(90);

	D_Presence = get_line_in(d_pin);  //get presence status 1=fail 0=present

	delay_us(300);

	D_SC = get_line_in(d_pin);  //check for short circuit

	if(D_SC == 0)  //if short circuit then error
		D_Error = 8;

	if(D_Presence == 1)   //if no device detected then error
		D_Error = 2;

	delay_us(10);
}



//**********************************************************************
void Write_byte(int8 Data)//write a byte to the DS18S20
{
	int8 count;

	for(count=8; count>0; count--)
	{
		D_Data= Data & 0x01; //LSB

		//write the bit to the port
		make_low(d_pin);

		delay_us(4); //time slot start

		//write data to the port
		if(D_Data)
			get_line_in(d_pin);
		   else
			make_low(d_pin);

		delay_us(66);

		//release port pin
		get_line_in(d_pin);

		delay_us(10);

		Data>>=1; //shift next bit into place
	}

	delay_us(10);
}



// tx[0] - number of bytes
// tx[1] - command

//********************************************************************/
void receive_packet()
{
	int8 tt, pn, pos, checksum, num_bytes, save_bytes;

	num_bytes=read_usb();//get the number of bytes
	buffer[0]=num_bytes;
	save_bytes = num_bytes;//save a copy
	pos=1;
	while(num_bytes--)
		buffer[pos++]=read_usb();

	checksum=0;
	for(pn=0; pn<save_bytes+1; pn++)
		checksum ^= (int8)buffer[pn];

	if(read_usb() == checksum)//if checksums match
	{
		delay_us(100);
		if(input(RXF)==0)//if more data in FIFO
		{
			//still have data in the FIFO -- ERROR!
			delay_us(5);
			while(input(RXF)==0)//clear the FIFO
				read_usb();

			send(0xAb);//error!
	     	packet_ready=0;
		}
       else
	   {
			send(0x55);//no error
			delay_us(100);
			packet_ready=1;
	   }
	}
	else//checksum failed
	{
		send(0xaa);//error!
		packet_ready=0;
	}

	
}


//********************************************************************/
void send(int8 thebyte)
{
	//send the byte back through the USB interface
	SET_TRIS_D( 0x0 );//all output

	while(input(TXE)==1);//when TXE is high do not write

	output_d(thebyte);
	delay_us(50);
    output_low(WR);//C5=0
	delay_us(20);
    output_high(WR);//C5=1

	SET_TRIS_D( 0xff );//all input
}



//********************************************************************/
int8 read_usb()
{
   int8 data8;

   while(input(RXF)==1);//wait for USB data to arrive

   //debug
	SET_TRIS_D( 0xff );//all input
	output_low(RD);//pull RD low
	delay_us(1);
    data8 = input_d();//read data
	output_high(RD);//take RD back high

   return data8;
}



/*
//////// Standard Header file for the PIC18F8722 device ////////////////
#device PIC18F8722
#nolist
//////// Program memory: 0x16  Data RAM: 3840  Stack: 31
//////// I/O: 40   Analog Pins: 16
//////// Data EEPROM: 1024
//////// C Scratch area: 00   ID Location: 2000
//////// Fuses: LP,XT,HS,RC,EC,EC_IO,H4,RC_IO,PROTECT,NOPROTECT,IESO,NOIESO
//////// Fuses: NOBROWNOUT,BROWNOUT,WDT1,WDT2,WDT4,WDT8,WDT16,WDT32,WDT64
//////// Fuses: WDT128,WDT,NOWDT,BORV25,BORV27,BORV42,BORV45,PUT,NOPUT,CPD
//////// Fuses: NOCPD,NOSTVREN,STVREN,NODEBUG,DEBUG,NOLVP,LVP,WRT,NOWRT,CPB
//////// Fuses: NOCPB,EBTRB,NOEBTRB,EBTR,NOEBTR,CCP2E7,CCP2C1,WRTD,NOWRTD
//////// Fuses: WRTC,NOWRTC,WRTB,NOWRTB,FCMEN,NOFCMEN,INTRC,INTRC_IO
//////// Fuses: BROWNOUT_SW,BROWNOUT_NOSL,WDT256,WDT512,WDT1024,WDT2048
//////// Fuses: WDT4096,WDT8192,WDT16384,WDT32768,LPT1OSC,NOLPT1OSC,MCLR
//////// Fuses: NOMCLR,XINST,NOXINST,MCU,CPU,CPU_BB,EMCU,WAIT,NOWAIT,BW16
//////// Fuses: BW8,ABW8,ABW12,ABW16,ABW20,ECCPE,ECCPH
//////// 
////////////////////////////////////////////////////////////////// I/O
// Discrete I/O Functions: SET_TRIS_x(), OUTPUT_x(), INPUT_x(),
//                         PORT_B_PULLUPS(), INPUT(),
//                         OUTPUT_LOW(), OUTPUT_HIGH(),
//                         OUTPUT_FLOAT(), OUTPUT_BIT()
// Constants used to identify pins in the above are:

#define PIN_A0  31744
#define PIN_A1  31745
#define PIN_A2  31746
#define PIN_A3  31747
#define PIN_A4  31748
#define PIN_A5  31749
#define PIN_A6  31750
#define PIN_A7  31751

#define PIN_B0  31752
#define PIN_B1  31753
#define PIN_B2  31754
#define PIN_B3  31755
#define PIN_B4  31756
#define PIN_B5  31757
#define PIN_B6  31758
#define PIN_B7  31759

#define PIN_C0  31760
#define PIN_C1  31761
#define PIN_C2  31762
#define PIN_C3  31763
#define PIN_C4  31764
#define PIN_C5  31765
#define PIN_C6  31766
#define PIN_C7  31767

#define PIN_D0  31768
#define PIN_D1  31769
#define PIN_D2  31770
#define PIN_D3  31771
#define PIN_D4  31772
#define PIN_D5  31773
#define PIN_D6  31774
#define PIN_D7  31775

#define PIN_E0  31776
#define PIN_E1  31777
#define PIN_E2  31778
#define PIN_E3  31779
#define PIN_E4  31780
#define PIN_E5  31781
#define PIN_E6  31782
#define PIN_E7  31783

#define PIN_F0  31784
#define PIN_F1  31785
#define PIN_F2  31786
#define PIN_F3  31787
#define PIN_F4  31788
#define PIN_F5  31789
#define PIN_F6  31790
#define PIN_F7  31791

#define PIN_G0  31792
#define PIN_G1  31793
#define PIN_G2  31794
#define PIN_G3  31795
#define PIN_G4  31796
#define PIN_G5  31797

#define PIN_H0  31800
#define PIN_H1  31801
#define PIN_H2  31802
#define PIN_H3  31803
#define PIN_H4  31804
#define PIN_H5  31805
#define PIN_H6  31806
#define PIN_H7  31807

#define PIN_J0  31808
#define PIN_J1  31809
#define PIN_J2  31810
#define PIN_J3  31811
#define PIN_J4  31812
#define PIN_J5  31813
#define PIN_J6  31814
#define PIN_J7  31815

////////////////////////////////////////////////////////////////// Useful defines
#define FALSE 0
#define TRUE 1

#define BYTE int
#define BOOLEAN short int

#define getc getch
#define fgetc getch
#define getchar getch
#define putc putchar
#define fputc putchar
#define fgets gets
#define fputs puts

////////////////////////////////////////////////////////////////// Control
// Control Functions:  RESET_CPU(), SLEEP(), RESTART_CAUSE()
// Constants returned from RESTART_CAUSE() are:

#define WDT_TIMEOUT       7    
#define MCLR_FROM_SLEEP  11    
#define MCLR_FROM_RUN    15    
#define NORMAL_POWER_UP  12    
#define BROWNOUT_RESTART 14    
#define WDT_FROM_SLEEP   3     
#define RESET_INSTRUCTION 0    

////////////////////////////////////////////////////////////////// Timer 0
// Timer 0 (AKA RTCC)Functions: SETUP_COUNTERS() or SETUP_TIMER_0(),
//                              SET_TIMER0() or SET_RTCC(),
//                              GET_TIMER0() or GET_RTCC()
// Constants used for SETUP_TIMER_0() are:
#define RTCC_INTERNAL   0
#define RTCC_EXT_L_TO_H 32
#define RTCC_EXT_H_TO_L 48

#define RTCC_DIV_1      8
#define RTCC_DIV_2      0
#define RTCC_DIV_4      1
#define RTCC_DIV_8      2
#define RTCC_DIV_16     3
#define RTCC_DIV_32     4
#define RTCC_DIV_64     5
#define RTCC_DIV_128    6
#define RTCC_DIV_256    7

#define RTCC_OFF        0x80  

#define RTCC_8_BIT      0x40  

// Constants used for SETUP_COUNTERS() are the above
// constants for the 1st param and the following for
// the 2nd param:

////////////////////////////////////////////////////////////////// WDT
// Watch Dog Timer Functions: SETUP_WDT() or SETUP_COUNTERS() (see above)
//                            RESTART_WDT()
//
#define WDT_ON      0x100   
#define WDT_OFF     0       

////////////////////////////////////////////////////////////////// Timer 1
// Timer 1 Functions: SETUP_TIMER_1, GET_TIMER1, SET_TIMER1
// Constants used for SETUP_TIMER_1() are:
//      (or (via |) together constants from each group)
#define T1_DISABLED         0
#define T1_INTERNAL         0x85
#define T1_EXTERNAL         0x87
#define T1_EXTERNAL_SYNC    0x83

#define T1_CLK_OUT          8

#define T1_DIV_BY_1         0
#define T1_DIV_BY_2         0x10
#define T1_DIV_BY_4         0x20
#define T1_DIV_BY_8         0x30

////////////////////////////////////////////////////////////////// Timer 2
// Timer 2 Functions: SETUP_TIMER_2, GET_TIMER2, SET_TIMER2
// Constants used for SETUP_TIMER_2() are:
#define T2_DISABLED         0
#define T2_DIV_BY_1         4
#define T2_DIV_BY_4         5
#define T2_DIV_BY_16        6

////////////////////////////////////////////////////////////////// Timer 3
// Timer 3 Functions: SETUP_TIMER_3, GET_TIMER3, SET_TIMER3
// Constants used for SETUP_TIMER_3() are:
//      (or (via |) together constants from each group)
#define T3_DISABLED         0
#define T3_INTERNAL         0x85
#define T3_EXTERNAL         0x87
#define T3_EXTERNAL_SYNC    0x83

#define T3_DIV_BY_1         0
#define T3_DIV_BY_2         0x10
#define T3_DIV_BY_4         0x20
#define T3_DIV_BY_8         0x30

////////////////////////////////////////////////////////////////// Timer 4
// Timer 4 Functions: SETUP_TIMER_4, GET_TIMER4, SET_TIMER4
// Constants used for SETUP_TIMER_4() are:
#define T4_DISABLED         0
#define T4_DIV_BY_1         4
#define T4_DIV_BY_4         5
#define T4_DIV_BY_16        6

////////////////////////////////////////////////////////////////// CCP
// CCP Functions: SETUP_CCPx, SET_PWMx_DUTY
// CCP Variables: CCP_x, CCP_x_LOW, CCP_x_HIGH
// Constants used for SETUP_CCPx() are:
#define CCP_OFF                         0
#define CCP_CAPTURE_FE                  4
#define CCP_CAPTURE_RE                  5
#define CCP_CAPTURE_DIV_4               6
#define CCP_CAPTURE_DIV_16              7
#define CCP_COMPARE_SET_ON_MATCH        8
#define CCP_COMPARE_CLR_ON_MATCH        9
#define CCP_COMPARE_INT                 0xA
#define CCP_COMPARE_INT_AND_TOGGLE      0x2       
#define CCP_COMPARE_RESET_TIMER         0xB
#define CCP_PWM                         0xC
#define CCP_PWM_PLUS_1                  0x1c
#define CCP_PWM_PLUS_2                  0x2c
#define CCP_PWM_PLUS_3                  0x3c
#define CCP_USE_TIMER3                  0x100       
long CCP_1;
#byte   CCP_1    =                      0xfbe       
#byte   CCP_1_LOW=                      0xfbe       
#byte   CCP_1_HIGH=                     0xfbf       
// The following should be used with the ECCP unit only (or these in)
#define CCP_PWM_H_H                     0x0c
#define CCP_PWM_H_L                     0x0d
#define CCP_PWM_L_H                     0x0e
#define CCP_PWM_L_L                     0x0f

#define CCP_PWM_FULL_BRIDGE             0x40
#define CCP_PWM_FULL_BRIDGE_REV         0xC0
#define CCP_PWM_HALF_BRIDGE             0x80

#define CCP_SHUTDOWN_ON_COMP1           0x100000
#define CCP_SHUTDOWN_ON_COMP2           0x200000
#define CCP_SHUTDOWN_ON_COMP            0x300000
#define CCP_SHUTDOWN_ON_INT0            0x400000
#define CCP_SHUTDOWN_ON_COMP1_INT0      0x500000
#define CCP_SHUTDOWN_ON_COMP2_INT0      0x600000
#define CCP_SHUTDOWN_ON_COMP_INT0       0x700000

#define CCP_SHUTDOWN_AC_L               0x000000
#define CCP_SHUTDOWN_AC_H               0x040000
#define CCP_SHUTDOWN_AC_F               0x080000

#define CCP_SHUTDOWN_BD_L               0x000000
#define CCP_SHUTDOWN_BD_H               0x010000
#define CCP_SHUTDOWN_BD_F               0x020000

#define CCP_SHUTDOWN_RESTART            0x80000000
#define CCP_DELAY                       0x1000000  // Multiply this by the delay count

long CCP_2;
#byte   CCP_2    =                      0xfbb       
#byte   CCP_2_LOW=                      0xfbb       
#byte   CCP_2_HIGH=                     0xfbc       
long CCP_3;
#byte   CCP_3    =                      0xfb8       
#byte   CCP_3_LOW=                      0xfb8       
#byte   CCP_3_HIGH=                     0xfb9       
long CCP_4;
#byte   CCP_4    =                      0xf74
#byte   CCP_4_LOW=                      0xf74
#byte   CCP_4_HIGH=                     0xf75
long CCP_5;
#byte   CCP_5    =                      0xf71
#byte   CCP_5_LOW=                      0xf71
#byte   CCP_5_HIGH=                     0xf72

// The following should be used with a call to SETUP_TIMER_3 (or one of these in)
#define  T3_CCP1_TO_5  0x48
#define  T3_CCP2_TO_5  0x8
#define  T3_CCP3_TO_5  0x40

////////////////////////////////////////////////////////////////// PSP
// PSP Functions: SETUP_PSP, PSP_INPUT_FULL(), PSP_OUTPUT_FULL(),
//                PSP_OVERFLOW(), INPUT_D(), OUTPUT_D()
// PSP Variables: PSP_DATA
// Constants used in SETUP_PSP() are:
#define PSP_ENABLED                     0x10
#define PSP_DISABLED                    0

#byte  PSP_DATA=    0xF83               

////////////////////////////////////////////////////////////////// SPI
// SPI Functions: SETUP_SPI, SPI_WRITE, SPI_READ, SPI_DATA_IN
// Constants used in SETUP_SSP() are:
#define SPI_MASTER       0x20
#define SPI_SLAVE        0x24
#define SPI_L_TO_H       0
#define SPI_H_TO_L       0x10
#define SPI_CLK_DIV_4    0
#define SPI_CLK_DIV_16   1
#define SPI_CLK_DIV_64   2
#define SPI_CLK_T2       3
#define SPI_SS_DISABLED  1

#define SPI_SAMPLE_AT_END 0x8000
#define SPI_XMIT_L_TO_H  0x4000

////////////////////////////////////////////////////////////////// UART
// Constants used in setup_uart() are:
// FALSE - Turn UART off
// TRUE  - Turn UART on
#define UART_ADDRESS           2
#define UART_DATA              4
#define UART_AUTODETECT        8
#define UART_AUTODETECT_NOWAIT 9
#define UART_WAKEUP_ON_RDA     10
////////////////////////////////////////////////////////////////// COMP
// Comparator Variables: C1OUT, C2OUT
// Constants used in setup_comparator() are:
#define F6_F5_F4_F5  0x70ff04
#define F6_F5_F4_F3_OUT_ON_F2_F1  0x78F903
#define F6_F5_F4_F5_OUT_ON_F2_F1  0x70F905
#define NC_NC_NC_NC  0x00ff07
#define F6_F5_F4_F3  0x78ff02
#define F6_F5_NC_NC_OUT_ON_F2  0x60FB01
#define F6_VR_F4_VR 0x50ff06
#define F5_VR_F3_VR 0x28ff0e

#bit C1OUT = 0xfb4.6
#bit C2OUT = 0xfb4.7

////////////////////////////////////////////////////////////////// VREF
// Constants used in setup_vref() are:
//
#define VREF_LOW  0xa0
#define VREF_HIGH 0x80
// Or (with |) the above with a number 0-15
#define VREF_F5   0x40
#define VREF_COMP 0x10

////////////////////////////////////////////////////////////////// VREF
// Constants used in setup_low_volt_detect() are:
//
#define LVD_LVDIN   0x1F
#define LVD_45 0x1E
#define LVD_42 0x1D
#define LVD_40 0x1C
#define LVD_38 0x1B
#define LVD_36 0x1A
#define LVD_35 0x19
#define LVD_33 0x18
#define LVD_30 0x17
#define LVD_28 0x16
#define LVD_27 0x15
#define LVD_25 0x14
#define LVD_23 0x13    
#define LVD_21 0x12    
#define LVD_19 0x11    

#define LVD_TRIGGER_BELOW 0
#define LVD_TRIGGER_ABOVE 0x80


////////////////////////////////////////////////////////////////// INTERNAL RC
// Constants used in setup_oscillator() are:
// First param:
#define OSC_31KHZ   0
#define OSC_125KHZ  0x10
#define OSC_250KHZ  0x20
#define OSC_500KHZ  0x30
#define OSC_1MHZ    0x40
#define OSC_2MHZ    0x50
#define OSC_4MHZ    0x60
#define OSC_8MHZ    0x70
#define OSC_16MHZ   0x4060
#define OSC_32MHZ   0x4070
// The following may be OR'ed in with the above using |
#define OSC_TIMER1  1
#define OSC_INTRC   2
#define OSC_NORMAL  0
// The following may be OR'ed in with the above using |
#define OSC_IDLE_MODE  0x80
#define OSC_31250   0x8000
#define OSC_PLL_ON  0x4000
#define OSC_PLL_OFF 0
// A second optional parameter may be used with this part to fine
// tune the speed (signed int,0-31)
// Result may be (ignore all other bits)
#define OSC_STATE_STABLE 4
#define OSC_STATE_EXT_RUNNING 8

////////////////////////////////////////////////////////////////// EXT MEM
// External Memory Functions: SETUP_EXTERNAL_MEMORY(), READ_EXTERNAL_MEMORY(),
//                            WRITE_EXTERNAL_MEMORY()
//
// Constants used in SETUP_EXTERNAL_MEMORY() are:
#define EXTMEM_BYTE_WRITE  0
#define EXTMEM_BYTE_SELECT 1
#define EXTMEM_WORD_WRITE  2
#define EXTMEM_DISABLE     0x80
// Use one of the above and optionaly OR in one of:
#define EXTMEM_WAIT_0 0x30
#define EXTMEM_WAIT_1 0x20
#define EXTMEM_WAIT_2 0x10
#define EXTMEM_WAIT_3 0x00  // Default


////////////////////////////////////////////////////////////////// ADC
// ADC Functions: SETUP_ADC(), SETUP_ADC_PORTS() (aka SETUP_PORT_A),
//                SET_ADC_CHANNEL(), READ_ADC()
// Constants used for SETUP_ADC() are:
#define ADC_OFF               0           // ADC Off
#define ADC_CLOCK_DIV_2   0x100
#define ADC_CLOCK_DIV_4    0x04
#define ADC_CLOCK_DIV_8    0x01
#define ADC_CLOCK_DIV_16   0x05
#define ADC_CLOCK_DIV_32   0x02
#define ADC_CLOCK_DIV_64   0x06
#define ADC_CLOCK_INTERNAL 0x07           // Internal 2-6us
// The following may be OR'ed in with the above using |
#define ADC_TAD_MUL_0      0x00
#define ADC_TAD_MUL_2      0x08
#define ADC_TAD_MUL_4      0x10
#define ADC_TAD_MUL_6      0x18
#define ADC_TAD_MUL_8      0x20
#define ADC_TAD_MUL_12     0x28
#define ADC_TAD_MUL_16     0x30
#define ADC_TAD_MUL_20     0x38

// Constants used in SETUP_ADC_PORTS() are:
#define NO_ANALOGS  0x0F  // None
#define ALL_ANALOG  0x00  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4 F5 F6 H4 H5 H6 H7 
#define AN0_TO_AN13 0x01  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4 F5 F6 H4 H5       
#define AN0_TO_AN12 0x02  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4 F5 F6 H4          
#define AN0_TO_AN11 0x03  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4 F5 F6             
#define AN0_TO_AN10 0x04  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4 F5                 
#define AN0_TO_AN9  0x05  // A0 A1 A2 A3 A5 F0 F1 F2 F3 F4                    
#define AN0_TO_AN8  0x06  // A0 A1 A2 A3 A5 F0 F1 F2 F3                       
#define AN0_TO_AN7  0x07  // A0 A1 A2 A3 A5 F0 F1 F2                         
#define AN0_TO_AN6  0x08  // A0 A1 A2 A3 A5 F0 F1
#define AN0_TO_AN5  0x09  // A0 A1 A2 A3 A5 F0
#define AN0_TO_AN4  0x0A  // A0 A1 A2 A3 A5
#define AN0_TO_AN3  0x0B  // A0 A1 A2 A3
#define AN0_TO_AN2  0x0C  // A0 A1 A2
#define AN0_TO_AN1  0x0D  // A0 A1
#define AN0         0x0E  // A0
#define ANALOG_AN0_TO_AN13 0x01  //!old only provided for compatibility   
#define ANALOG_AN0_TO_AN12 0x02  //!old only provided for compatibility   
#define ANALOG_AN0_TO_AN11 0x03  //!old only provided for compatibility   
#define ANALOG_AN0_TO_AN10 0x04  //!old only provided for compatibility   
#define ANALOG_AN0_TO_AN9  0x05  //!old only provided for compatibility   
#define ANALOG_AN0_TO_AN8  0x06  //!old only provided for compatibility   
#define ANALOG_AN0_TO_AN7  0x07  //!old only provided for compatibility   
#define ANALOG_AN0_TO_AN6  0x08  //!old only provided for compatibility
#define ANALOG_AN0_TO_AN5  0x09  //!old only provided for compatibility
#define ANALOG_AN0_TO_AN4  0x0A  //!old only provided for compatibility
#define ANALOG_AN0_TO_AN3  0x0B  //!old only provided for compatibility
#define ANALOG_AN0_TO_AN2  0x0C  //!old only provided for compatibility
#define ANALOG_AN0_TO_AN1  0x0D  //!old only provided for compatibility
#define ANALOG_AN0         0x0E  //!old only provided for compatibility

// The following may be OR'ed in with the above using |
#define VSS_VDD               0x00              // Range 0-Vdd
#define VREF_VREF             0x30              // Range VrefL-VrefH
#define VREF_VDD              0x20              // Range VrefL-Vdd
#define VSS_VREF              0x10              // Range 0-VrefH



// Constants used in READ_ADC() are:
#define ADC_START_AND_READ     7   // This is the default if nothing is specified
#define ADC_START_ONLY         1
#define ADC_READ_ONLY          6



////////////////////////////////////////////////////////////////// INT
// Interrupt Functions: ENABLE_INTERRUPTS(), DISABLE_INTERRUPTS(),
//                      EXT_INT_EDGE()
//
// Constants used in EXT_INT_EDGE() are:
#define L_TO_H              0x40
#define H_TO_L                 0
// Constants used in ENABLE/DISABLE_INTERRUPTS() are:
#define GLOBAL                    0xF2C0
#define INT_RTCC                  0xF220
#define INT_TIMER0                0xF220
#define INT_TIMER1                0x9D01
#define INT_TIMER2                0x9D02
#define INT_TIMER3                0xA002
#define INT_EXT                   0xF210
#define INT_EXT1                  0xF008
#define INT_EXT2                  0xF010
#define INT_RB                    0xF208
#define INT_PSP                   0x9D80
#define INT_AD                    0x9D40
#define INT_RDA                   0x9D20
#define INT_TBE                   0x9D10
#define INT_SSP                   0x9D08
#define INT_CCP1                  0x9D04
#define INT_CCP2                  0xA001
#define INT_BUSCOL                0xA008
#define INT_LOWVOLT               0xA004
#define INT_COMP                  0xA040
#define INT_EEPROM                0xA010
#define INT_RDA2                  0xA320
#define INT_TBE2                  0xA310
#define INT_TIMER4                0xA308
#define INT_CCP3                  0xA301
#define INT_CCP4                  0xA302
#define INT_CCP5                  0xA304
#define INT_OSCF                  0xA080
#define INT_BUSCOL2               0xA340
#define INT_SSP2                  0xA380
#define INT_EXT3                  0xF020

#list


*/
